#include "error.h"
#include <float.h>
#include <iomanip.h>
#include "interval.h"
#include "lineInterval.h"
#include "secondDerive.h"
#include "taylorInterval.h"
#include "recurse.h"

/**
  * Interval arithmetic for Hales-McLaughlin "Proof of the Dodecahedral Conjecture"
  * The only inequalites here are those added in the Nov 2001 proof.
  * These inequalities were run on a WINDOWS operating system.
  */

void selfTest()
	{
	interMath::selfTest();
	linearization::selfTest();
	secondDerive::selfTest();
	taylorFunction::selfTest();
	}

void miniTest() {
	cout.precision(30);
	interval A("2.1");
	interval B("2.3");
	interval C = A*B;
	cout << C << endl;
	cout << interMath::atan(C) << endl;
	double x = 1.1;
	double y = 1.1;
	cout << "none " << x*y << endl;

	unsigned t;
	t = _control87(RC_DOWN,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(RC_UP,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(0x0800,MCW_RC);
	cout << t << " " <<  x*y << endl;
	x = 0.9;
	y = 0.9;
	t = _control87(0x0000,MCW_RC);
	cout.precision(16);
	cout << t << " " << x*y << endl;
	t = _control87(0x0400,MCW_RC);
	cout << t << " " << x*y << endl;
	t = _control87(0x0800,MCW_RC);
	cout << t << " " <<  x*y << endl;
	ROUND_UP;
	cout << x*y << endl;
	ROUND_DOWN;
	cout << x*y << endl;
}

//
// prove that dih < 1.153093 of a flat quarter.
int testRun()
	{
	interval tx[6]={"4","4","4","6.3001","4","4"};
	interval tz[6]={"6.3001","6.3001","6.3001","6.3001","6.3001","6.3001"};
	domain x = domain::lowerD(tx);
	domain z = domain::upperD(tz);
	taylorFunction F = taylorSimplex::dih*"-1"+taylorSimplex::unit*"1.153093";
	F.setReducibleState(1);
	return prove::generic (x,z,F);
	}

// prove that 0 > coeffdih*dih +coeffsol*sol + a + b1 y1 + b2 y2 + ... b6 y6
// When xi varies from xmi to xMi.
// The values are the exact double values of the inequalities requested.
int proveDihSol(char* coeffdih, char* coeffsol, char* a, 
			char* b1,char* b2,char* b3,char* b4, char* b5,char* b6,
			char* xm1, char* xm2, char* xm3, char* xm4, char* xm5, char* xm6,
			char* xM1, char* xM2, char* xM3, char* xM4, char* xM5, char* xM6) {
	 if (!INTERVAL_ON) 
		 error::message("interval arithmetic turned off");
	interval C(coeffdih);
	interval Csol(coeffsol);
	interval tx[6]={xm1,xm2,xm3,xm4,xm5,xm6};
	interval tz[6]={xM1,xM2,xM3,xM4,xM5,xM6};
	interval zero("0");
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	int bdih = interMath::sup(C)!=0 || interMath::inf(C)!=0;
	int bsol =  interMath::sup(Csol)!=0 || interMath::inf(Csol)!=0;
	taylorFunction F = 
		(bdih ? taylorSimplex::dih*C : taylorSimplex::unit*zero) 
		+ (bsol ? taylorSimplex::sol*Csol : taylorSimplex::unit*zero)
		+taylorSimplex::unit*a
		+taylorSimplex::y1*b1
		+taylorSimplex::y2*b2
		+taylorSimplex::y3*b3
		+taylorSimplex::y4*b4
		+taylorSimplex::y5*b5
		+taylorSimplex::y6*b6;
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; //  add this in release version.
	return 1;
}

// When xi varies from xmi to xMi.
// The values are the exact double values of the inequalities requested.
int proveQuoin(char* a, 
			char* b1,char* b2,char* b6,
			char* xm1, char* xm2, char* xm6,
			char* xM1, char* xM2, char* xM6) {
	 if (!INTERVAL_ON) 
		 error::message("interval arithmetic turned off");
	interval tx[6]={xm1,xm2,"4","4","4",xm6};
	interval tz[6]={xM1,xM2,"4","4","4",xM6};
	interval zero("0");
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::quoDodec*"-1"
		+taylorSimplex::unit*a
		+taylorSimplex::y1*b1
		+taylorSimplex::y2*b2
		+taylorSimplex::y6*b6;
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; //  add this in release version.
	return 1;
}

// When xi varies from xmi to xMi.
// The values are the exact double values of the inequalities requested.
int proveMu() { 
	char* seanX = "6.3343829124";
	char* vx = "8.1225"; // 2.85^2;
	interval tx[6]={"4","4","4",vx,"4","4"};
	interval tz[6]={seanX,seanX,seanX,vx,seanX,seanX};
	//interval doct("0.7209029495174650");
	//interval seanM("0.42755");
	interval vorVcCoeff(" -0.346787317443127380"); // -1/(4 doct)
	interval solCoeff("0.0348330899241698410"); // 1/(3 doct) - seanM 
	interval zero("0");
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::VorVcSeanT*vorVcCoeff*"-1" + taylorSimplex::sol*solCoeff*"-1"
		+taylorSimplex::unit*"0.02";
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; 
	return 1;
}

// When an edge >= 2.3, Rad[2,2,2,2.3,2,2] > seanT, so volan > vol(*,seanT), and the inequality is true.
int prove0051() { 
	char* v23 = "5.29"; // 2.3^2;
	interval tx[6]={"4","4","4","4","4","4"};
	interval tz[6]={v23,v23,v23,v23,v23,v23};
	interval doct("0.7209029495174650");
	interval mdoct4_00051 = doct*"-4"*"0.005";
	//interval mdoct4_00051("-32.0"); // test case.
	interval zero("0");
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::VorVcSeanT*"-1" + taylorSimplex::vor
		+taylorSimplex::unit*mdoct4_00051;
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; 
	return 1;
}

// 
int prove2dihA() { 
	char* seanX = "6.3343829124";
	interval tx[6]={"4","4","4","4","4","4"};
	interval tz[6]={seanX,seanX,seanX,"4","4","8"};
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::dih_45 + taylorSimplex::dih_345*"-1" + taylorSimplex::unit*"-0.1";
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; 
	return 1;
}

// 
int prove2dihB() { 
	char* seanX = "6.3343829124";
	interval tx[6]={"4","4","4","10.24","8.4681"/*2.91^2*/,"4"};
	interval tz[6]={seanX,seanX,seanX,"10.24","10.24","8"};
	domain x = domain::lowerD(tx);
	domain z = domain::lowerD(tz);
	taylorFunction F = taylorSimplex::dih*"-1" + taylorSimplex::dih_345 + taylorSimplex::unit*"0.1";
	F.setReducibleState(0);
	if (!prove::generic (x,z,F)) return 0;
	if (error::getCount()>0) return 0; 
	return 1;
}

int main(int argc, char* argv[])
	{
	//miniTest();
	selfTest();
	cout.precision(20);
	if (testRun()) 
		cout << "YES!"; else cout << "NO!" ;
	cout << flush;
	char* seanX = "6.3343829124";
	char* vx = "8.1225"; // 2.85^2;
	char* v32 = "10.24"; // 3.2^2
	/*
	// #1:
	if (proveQuoin("0.0735","-0.01","-0.0135","-0.0135",
			"4","4","4",seanX,seanX,seanX))
		cout << "YES"; else cout << "NO";
	cout << " #1\n" << flush;
	// #2:
	if (proveDihSol("-1.0","0","2.384","0.23","-0.47","-0.47","0.7","-0.285","-0.285",
			"4","4","4","4","4","4",seanX,seanX,seanX,seanX,seanX,seanX))
		cout << "YES #2"; else cout << "NO #2";
	cout << "\n" << flush;
	// #3:
	if (proveDihSol("1.0","0","0.134","-0.54","0.15","0.15","-0.76","0.156","0.156",
			"4","4","4","4","4","4",seanX,seanX,seanX,seanX,seanX,seanX))
		cout << "YES #3"; else cout << "NO #3";
	cout << "\n" << flush;
	// #4:
	if (proveDihSol("1.0","0","-0.164","-0.544","0.169","0.169","-0.706","0.18","0.18",
			"4","4","4","4","4","4",seanX,seanX,seanX,seanX,seanX,seanX))
		cout << "YES #4"; else cout << "NO #4";
	cout << "\n" << flush;
	// #5:
	if (proveDihSol("0","-1.0","1.7911","-0.401","-0.401","-0.401","0.194","0.194","0.194",
			"4","4","4","4","4","4",seanX,seanX,seanX,seanX,seanX,seanX))
		cout << "YES #5"; else cout << "NO #5";
	cout << "\n" << flush;
	// #6:
	if (proveDihSol("-1.0","0","4.129","0.328","-0.396","-0.396","0","-0.396","-0.396",
			"4","4","4",seanX,"4","4",seanX,seanX,seanX,seanX,seanX,seanX))
		cout << "YES #6"; else cout << "NO #6";
	cout << "\n" << flush;
	return 0;
	// #7:
	if (proveDihSol("-1.0","0","4.906","0.335","-0.51","-0.51","0","-0.404","-0.404",
			"4","4","4",vx,"4","4",seanX,seanX,seanX,vx,seanX,seanX))
		cout << "YES"; else cout << "NO";
	cout << " #7\n" << flush;
	// #8:
	if (proveDihSol("1.0","0","-0.2352409","-0.362","0.225","0.088","-0.7068","0.381","-0.1495",
			"4","4","4","4",seanX,"4",seanX,seanX,seanX,seanX,vx,seanX))
		cout << "YES"; else cout << "NO";
	cout << " #8\n" << flush;
	return 0;
	// #9:
	if (proveDihSol("-1.0","0","2.778","0.112","-0.679","-0.3485","0.606","-0.428","-0.004",
			"4","4","4","4",seanX,"4",seanX,seanX,seanX,seanX,vx,seanX))
		cout << "YES"; else cout << "NO";
	cout << " #9\n" << flush;
	// #10:
	if (prove0051())
		cout << "YES"; else cout << "NO";
	cout << " #10\n" << flush;
	// #11:
	if (proveMu())
		cout << "YES"; else cout << "NO";
	cout << " #11\n" << flush;
	// #1A:
	char* v27 = "7.29";//2.7^2
	if (proveDihSol("-1.0","0","1.2","0","0","0","0","0","0",
			"4","4","4",v32,"4",v27,seanX,seanX,seanX,v32,v32,"8"))
		cout << "YES"; else cout << "NO";
	cout << " #1A\n" << flush;
	*/


	// #3A:
	cout << "2dihA:\n";
	if (prove2dihA())
		cout << "YES"; else cout << "NO";
	cout << " #3A\n" << flush;
	// #4A:
	if (prove2dihB())
		cout << "YES"; else cout << "NO";
	cout << " #4A\n" << flush;
	return 1;
	}
